shock = {} -- Holds functions
shocks = {} -- Holds shockwaves

function love.load()
	love.graphics.setBackgroundColor(128,128,128) -- Background
	image = love.graphics.newImage("screenshot3.png") -- Background image

	shader = love.graphics.newShader("shader.glsl") -- Circle generation shader
	shader2 = love.graphics.newShader("shader2.glsl") -- Displacement shader
	canvas = love.graphics.newCanvas(1920,1080) -- Displacement canvas
	blank = love.graphics.newCanvas(1000,1000) -- Blank canvas for drawing circles
	on = true -- shader2 state
end

function shock.add(x,y,w,f) -- adds a new shockwave
	table.insert(shocks,{x=x,y=y,w=w,f=f,t=0})
end

function shock.update(dt)
	if (#shocks ~= 0) then -- Don't run the code if there's no shockwaves
		love.graphics.setCanvas(canvas) -- Setting the displacement canvas
		love.graphics.setShader(shader) -- setting the drawing shader
		love.graphics.clear() -- Clearing the canvas
		for i,v in pairs(shocks) do
			v.t = v.t + dt*1000 -- updates shockwave's lifetime -- lower this value for slow motion effect
			shader:send("time", v.t) -- sends time to shader
			shader:send("sfunction", v.f) -- sends some variables to shader
			shader:send("swidth", v.w) -- sends some variables to shader
			shader:send("fall", 1-((v.t+v.w)/500)^6) -- sends alpha to shader
			love.graphics.setBlendMode("screen","premultiplied") -- Some blend mode
			love.graphics.draw(blank,v.x,v.y,0,1,1,500,500) -- the actual drawing
			love.graphics.setBlendMode("alpha")
			if v.t > 500 then -- removal of old shockwaves
				table.remove(shocks,i)
			end
		end
		love.graphics.setShader()
		love.graphics.setCanvas()
	end
end

function love.update(dt)
	shock.update(dt)
	local x,y = love.mouse.getPosition()
	if love.keyboard.isDown("space") then
		shock.add(x,y,50,0.16)
	end
end


function love.draw()
	if on == true then
		love.graphics.setShader(shader2) -- seting displacement shader
		shader2:send("canvas",canvas) -- sending the displacement canvas to shader
	end
	love.graphics.draw(image) -- drawing background image
	if on == true then
		love.graphics.setShader()
	else
		love.graphics.draw(canvas)
	end
	love.graphics.print("FPS: " .. love.timer.getFPS(), 10, 10)
	love.graphics.print("#shocks: " .. #shocks, 10, 20)
	for i,v in pairs(shocks) do
		love.graphics.print("[" .. i .. "]time: " .. string.format("%5.2f",v.t), 10, 30+i*20)
	end
end


function love.mousepressed(x,y,button)
	shock.add(x,y,50,0.16)
end

function love.mousereleased(x,y,button)

end

function love.keypressed( key )
	if key == "r" then
		on = not on
	end
end
